using System;
using System.Text;
using System.Web.UI.WebControls;
using System.Collections.Generic;

using CMS.GlobalHelper;
using CMS.UIControls;

public partial class CMSAdminControls_UI_UniMenu_UniMenuButtons : CMSUserControl
{
    #region "Variables"

    /// <summary>
    /// Buttons
    /// </summary>
    private string[,] mButtons = null;

    private bool mAllowSelection = false;

    private int identifier = 0;

    protected int mSelectedIndex = 0;

    private List<Panel> mInnerControls = null;

    #endregion


    #region "Constants"

    public const string SelectedSuffix = " Selected";

    #endregion


    #region "Public properties"

    /// <summary>
    /// Determines whether buttons are selectable
    /// </summary>
    public bool AllowSelection
    {
        get
        {
            return mAllowSelection;
        }
        set
        {
            mAllowSelection = value;
        }
    }


    /// <summary>
    /// If selection is enabled, determines which button is implicitly selected
    /// </summary>
    public int SelectedIndex
    {
        get
        {
            return mSelectedIndex;
        }
        set
        {
            mSelectedIndex = value;
        }
    }

    #endregion


    #region "Properties"

    /// <summary>
    /// Array of buttons. 
    /// {
    /// [n, 0] - Caption of button
    /// [n, 1] - Button tooltip (string.empty: empty, null: copies caption or custom string)
    /// [n, 2] - CSS class of button
    /// [n, 3] - JavaScript OnClick action
    /// [n, 4] - Button's redirect URL
    /// [n, 5] - Image path of button
    /// [n, 6] - Image alternate text (string.empty: empty, null: copies caption or custom string
    /// [n, 7] - ImageAlign enum (default: NotSet)
    /// }
    /// </summary> 
    public string[,] Buttons
    {
        get
        {
            return mButtons;
        }
        set
        {
            mButtons = value;
        }
    }


    /// <summary>
    /// Description.
    /// </summary>
    public List<Panel> InnerControls
    {
        get
        {
            return mInnerControls;
        }
        set
        {
            mInnerControls = value;
        }
    }

    #endregion


    #region "Methods"

    protected void Page_Load(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
        }
        else
        {
            if (Buttons != null)
            {
                if (AllowSelection)
                {
                    ScriptHelper.RegisterJQuery(Page);
                    StringBuilder selectionScript = new StringBuilder();
                    selectionScript.Append("function SelectButton(elem)");
                    selectionScript.Append("{");
                    selectionScript.Append("    var selected = '" + SelectedSuffix + "';");
                    selectionScript.Append("    var jElem =$j(elem);");
                    selectionScript.Append("    jElem.siblings().removeClass(selected);");
                    selectionScript.Append("    jElem.addClass(selected);");
                    selectionScript.Append("}");
                    ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "selectionScript", ScriptHelper.GetScript(selectionScript.ToString()));

                    StringBuilder indexSelection = new StringBuilder();
                    indexSelection.Append("function SelectButtonIndex_" + ClientID + "(index)");
                    indexSelection.Append("{");
                    indexSelection.Append("    var elem = document.getElementById('" + ClientID + "_pnlButton'+index);");
                    indexSelection.Append("    SelectButton(elem);");
                    indexSelection.Append("");
                    indexSelection.Append("}");
                    ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "indexSelection_" + ClientID, ScriptHelper.GetScript(indexSelection.ToString()));
                }

                // For RTL (switches order of groups)
                int rtlParameter = CultureHelper.IsUICultureRTL() ? Buttons.GetUpperBound(0) : 0;

                int buttonsCount = Buttons.GetUpperBound(0) + 1;
                InnerControls = new List<Panel>(buttonsCount);
                for (identifier = 0; identifier < buttonsCount; identifier++)
                {
                    // Check array dimensions
                    if (Buttons.GetUpperBound(1) != 7)
                    {
                        Controls.Add(GetError(ResHelper.GetString("unimenubuttons.wrongdimensions")));
                        continue;
                    }
                    // For RTL
                    int realIndex = CultureHelper.IsUICultureRTL() ? (rtlParameter - identifier) : identifier;

                    string caption = Buttons[realIndex, 0];
                    string tooltip = Buttons[realIndex, 1];
                    string cssClass = Buttons[realIndex, 2];
                    string onClick = Buttons[realIndex, 3];
                    string redirectUrl = Buttons[realIndex, 4];
                    string imagePath = Buttons[realIndex, 5];
                    string alt = Buttons[realIndex, 6];
                    string align = Buttons[realIndex, 7];
                    ImageAlign imageAlign = ImageAlign.NotSet;

                    if (align != null)
                    {
                        try
                        {
                            // Try to get image align
                            imageAlign = (ImageAlign)Enum.Parse(typeof(ImageAlign), align);

                            // RTL switch
                            if (CultureHelper.IsUICultureRTL())
                            {
                                if (imageAlign == ImageAlign.Left)
                                {
                                    imageAlign = ImageAlign.Right;
                                }
                                else if (imageAlign == ImageAlign.Right)
                                {
                                    imageAlign = ImageAlign.Left;
                                }
                            }
                        }
                        catch
                        {
                            imageAlign = ImageAlign.NotSet;
                        }
                    }
                    else
                    {
                        imageAlign = ImageAlign.NotSet;
                    }

                    // Generate button image
                    Image buttonImage = new Image();
                    buttonImage.ID = "imgButton" + realIndex;
                    buttonImage.EnableViewState = false;
                    buttonImage.AlternateText = alt ?? caption;
                    if (!string.IsNullOrEmpty(imagePath))
                    {
                        buttonImage.ImageUrl = ResolveUrl(imagePath);
                    }
                    buttonImage.ImageAlign = imageAlign;

                    // Generate button text
                    Literal captionLiteral = new Literal();
                    captionLiteral.ID = "ltlCaption" + realIndex;
                    captionLiteral.EnableViewState = false;
                    captionLiteral.Text = "\n" + caption;

                    // Generate button link
                    HyperLink buttonLink = new HyperLink();
                    buttonLink.ID = "lnkButton" + realIndex;
                    buttonLink.EnableViewState = false;
                    buttonLink.Controls.Add(buttonImage);
                    buttonLink.Controls.Add(captionLiteral);

                    if (!string.IsNullOrEmpty(redirectUrl))
                    {
                        buttonLink.NavigateUrl = ResolveUrl(redirectUrl);
                    }

                    // Generate whole button
                    Panel pnlButton = new Panel();
                    pnlButton.ID = "pnlButton" + realIndex;
                    pnlButton.EnableViewState = false;
                    if (AllowSelection && realIndex == SelectedIndex)
                    {
                        cssClass += SelectedSuffix;
                    }
                    pnlButton.CssClass = cssClass;
                    pnlButton.Controls.Add(buttonLink);
                    pnlButton.ToolTip = tooltip ?? caption;
                    if (AllowSelection)
                    {
                        onClick = "SelectButton(this);" + onClick;
                    }
                    pnlButton.Attributes["onclick"] = onClick;

                    // Add to result
                    Controls.Add(pnlButton);

                    InnerControls.Insert(identifier, pnlButton);
                }
            }
            else
            {
                Controls.Add(GetError(ResHelper.GetString("unimenubuttons.wrongdimensions")));
            }
        }
    }

    #endregion


    #region "Other methods"

    /// <summary>
    /// Generates error label
    /// </summary>
    /// <param name="message">Error message to display</param>
    /// <returns>Label with error message</returns>
    protected Label GetError(string message)
    {
        // If error occures skip this group
        Label errorLabel = new Label();
        errorLabel.ID = "lblError" + identifier;
        errorLabel.EnableViewState = false;
        errorLabel.Text = message;
        errorLabel.CssClass = "ErrorLabel";
        return errorLabel;
    }

    #endregion
}
